#include "Vista.h"
	SUBROUTINE ANNULUS(A,IMAGESR,IMAGEER,IMAGESC,IMAGEEC)

********************************************************************************
*                                                                              *
*                            VISTA ANNULUS COMMAND                             *
*                                                                              *
*   Generates a radial brightness profile from an image by azimuthal averaging *
*   over a series of concentric annuli.  The annuli may be given an angle      *
*   of inclination and major axis position angle, where by the annuli are      *
*   then concentric ellipses, otherwise by default the annuli are round.       *
*                                                                              *
*   The user must first either run AXES to compute the image centroid for the  *
*   annuli, or otherwise compute a centroid and enter it by hand via an        *
*   optional keyword                                                           *
*                                                                              *
*   There is a provision to allow the user to use contours generated by the    *
*   PROFILE command for the azimuthal averaging via a keyword, otherwise,      *
*   ANNULUS provides a quick and dirty way of finding an average radial        *
*   profile of an object without taking into account actual isophotal contours *
*   at all.                                                                    *
*                                                                              *
*   COMMAND SYNTAX                                                             *
*                                                                              *
*       ANNULUS dest source N=n [STEP=dr] [PA=pa] [INC=i] [CEN=r0,c0]          *
*               [SCALE=s] [FAST] [PROF] [RAD=r]                                *
*                                                                              *
*   where:                                                                     *
*                                                                              *
*       dest            Buffer to hold the average radial profile (spectrum)   *
*                                                                              *
*       source          Buffer with the image                                  *
*                                                                              *
*       N=n             Number of annuli to use starting at center             *
*                                                                              *
*       STEP=dr         (optional) Spacing between annuli.  If no scale is     *
*                       given with the "SCALE=" keyword, the annulus spacing   *
*                       is in units of pixels.                                 *
*                       The default value is 1.0 (single pixel steps).         *
*                                                                              *
*       PA=pa           (optional) Position Angle of Major Axis in degrees     *
*                       measured Counterclockwise from the top of the image.   *
*                       The default value is 0.0 (circular annuli).            *
*                       If the optional PROF keyword is used, this keyword is  *
*                       ignored.                                               *
*                                                                              *
*       INC=i           (optional) Inclination of Annuli in degrees defined as *
*                       follows:                                               *
*                               Face-On : i = 0 degrees                        *
*                               Edge-On : i = 90 degrees                       *
*                       i must be less than 90 degrees.                        *
*                       The default value is 0.0 (circular annuli).            *
*                       If the optional PROF keyword is used, this keyword is  *
*                       ignored.                                               *
*                                                                              *
*       CEN=r0,c0       (optional) Center of annuli (in rows,colums on image). *
*                       If not given, the program will use the object centroid *
*                       computed by the most recent AXES command.  If AXES has *
*                       not been run prior to running ANNULUS, the program     *
*                       stop.                                                  *
*                       If the optional PROF keyword is used, this keyword is  *
*                       ignored.                                               *
*                                                                              *
*       SCALE=s         (optional) Image scale, typically expressed in units   *
*                       of arcseconds/pixel.  The default value of s is 1.0.   *
*                                                                              *
*       FAST            (optional) Use faster bi-linear interpolation rather   *
*                       than slower but more accurate Sinc interpolation.      *
*                       Results will be screwy for small radii, so be warned.  *
*                                                                              *
*       PROF            (optional) Computes averages along contours defined by *
*                       the contents of the PROFILE common block.  This uses   *
*                       previously derived elliptical isophots and averages    *
*                       along them.  The user must have previously used the    *
*                       PROFILE command, or read in a SAVE`d profile with the  *
*                       GET command.                                           *
*                                                                              *
*       RAD=r           (optional) Allows the user to compute an average along *
*                       a single annulus.  In this case, the "dest" buffer     *
*                       number is not required, or ignored if given.           *
*                                                                              *
* ---------------------------------------------------------------------------- *
*                                                                              *
*   Author:   Rick Pogge     1987 March 13                                     *
*                                                                              *
*             Lick Observatory                                                 *
*             University of California                                         *
*             Santa Cruz, CA  95064                                            *
*                                                                              *
*   Compatability:   AXES and PROFILE                                          *
*                    Output Redirection allowed                                *
*                    VISTA version 3                                           *
*                                                                              *
*   Modification History:                                                      *
*                                                                              *
*                                                                              *
********************************************************************************

*   The image to plot:

	REAL*4  A(IMAGESC:IMAGEEC,IMAGESR:IMAGEER)

*   Communication with VISTA
#ifdef VMS
	INCLUDE 'VINCLUDE:VISTALINK.INC'
	INCLUDE 'VINCLUDE:IMAGELINK.INC'
	INCLUDE 'VINCLUDE:CUSTOMIZE.INC'
	INCLUDE 'VINCLUDE:PROFILE.INC'
#else
	INCLUDE 'vistadisk/source/include/vistalink.inc'
	INCLUDE 'vistadisk/source/include/imagelink.inc'
	INCLUDE 'vistadisk/source/include/customize.inc'
	INCLUDE 'vistadisk/source/include/profile.inc'
#endif
*   The AXES common block

	REAL*4  ECCAX
C                               ! eccentricity from AXES
	REAL*4  ANGAX
C                               ! PA of principle axis from AXES
	REAL*4  ANGMAJAX
C                               ! PA of major axis from AXES
	REAL*4  ANGMINAX
C                               ! PA of minor axis from AXES
	REAL*4  XCAX, YCAX
C                               ! centroid (C,R) from AXES
	REAL*4  XPAX, YPAX
C                               ! position (C,R) of peak pixel from AXES

	COMMON /AX/ ECCAX, ANGAX, ANGMAJAX, ANGMINAX, XCAX, YCAX,
     .          XPAX, YPAX

*   Previous PROFILE calculations are held in the PRF common block.  The
*   parameters kept for each contour are as follows:
*
*               1       Contour central row number
*               2       Contour central column number
*               3       Position angle of contour, N at image top
*               4       Eccentricty of contour
*               5       Average surface brightness of contour
*               6       N=4 sine component of contour (average)
*               7       N=4 cosine component
*               8       N=2 sine component
*               9       N=2 cosine component
*              10       N=1 sine component
*              11       N=1 cosine component
*              12       Contour intensity derivative
*       NPRF           The number of contours
*       SCALE           The image scale ''/pixel

	INTEGER         ANNMAX

	PARAMETER (ANNMAX = 500)
C    The maximum number of annuli

C   Variables
	REAL*4          AZAVG(ANNMAX)
	COMMON /WORK/ AZAVG
C    array of azimuthal averages.
	REAL*4          B
C    semi-minor axis of ellipse.
	REAL*4          C0
C    center of annuli in Columns.
	REAL*4          COSTH
C    cosine of azimuthal angle.
	REAL*4          COSINC
C    cosine of inclination angle ( = ellipticity).
	REAL*4          CSAMP
C    column of sampling point.
	REAL*4          DCON
C    radians per degree.
	REAL*8          DELT1
C    FITS header axis scale card
	REAL*4          DRANN
C    spacing between annuli in pixels.
	REAL*4          DTHETA
C    step in azimuthal angle.
	REAL*4          ECC
C    eccentricity of an annulus.
	REAL*4          F
C    generic real variable.
	REAL*4          G(2)
C    generic real array.
	REAL*4          INC
C    inclination angle of annuli (in degrees).
	INTEGER         ICS
C    integer form of CSAMP.
	INTEGER         IRS
C    integer form of RSAMP.
	REAL*4          ISAMP
C    data value at sampling point.
	REAL*4          ISCALE
C    image scale (units/pixel).
	INTEGER         NACTUAL
C    number of samples taken.
	INTEGER         NANN
C    number of annuli.
	INTEGER         NCOL
C    number of image columns.
	INTEGER         NROW
C    number of image rows.
	INTEGER         NSAMP
C    number of sampling points along each annulus.
	REAL*4          PA
C    position angle of annuli (in degrees).
	CHARACTER       PARM*80
C    for reading keyword params.
	REAL*4          PHI
C    azimuthal angle from major axis (radians).
	REAL*4          PI
	REAL*4          PLOST
C    % of a given annulus lying inside image bounds.
	REAL*4          R0
C    center of annuli (rows).
	REAL*4          RANN
C    radius of an annulus.
	REAL*4          RPROJ
C    projected radius of annulus for a given THETA and INC.
	REAL*4          RSAMP
C    row of sampling point.
	REAL*4          SININC
C    sine of inclination angle ( = eccentricity).
	REAL*4          SINTH
C    sine of azimuthal angle.
	REAL*4          THETA
C    Azimuthal angle measured from annulus major axis.
C    Logical flags
C     a center has been defined.
	LOGICAL         CENTER
C     use faster bilinear interpolation on outer annuli
	LOGICAL         FAST
	LOGICAL         KEYCHECK
C     use previous PROFILE run.
	LOGICAL         PROF
C     user defined image scale.
	LOGICAL         SCALED
C     use only one annulus.
	LOGICAL         SINGLE
C     user defined annular spacing.
	LOGICAL         STEP
        LOGICAL         SILENT

*   Check keywords.  Exit if any are not understood.

	CALL KEYINIT
	CALL KEYDEF ('N=')
	CALL KEYDEF ('STEP=')
	CALL KEYDEF ('PA=')
	CALL KEYDEF ('INC=')
	CALL KEYDEF ('CEN=')
	CALL KEYDEF ('SCALE=')
	CALL KEYDEF ('RAD=')
	CALL KEYDEF ('FAST')
	CALL KEYDEF ('PROF')
	CALL KEYDEF ('PROF=')
        CALL KEYDEF ('SILENT')

*   If the user has given a keyword that is not understood, exit, but also
*   print a command syntax reminder
	IF (.NOT. KEYCHECK()) THEN
	    XERR = .TRUE.
	    PRINT *,' USAGE:'
	    PRINT *,'   ANNULUS dest source N=n [STEP=dr] [CEN=r0,c0] [PA=pa]'
	    PRINT *,'           [INC=i] [SCALE=s] [FAST] [PROF] [RAD=r]'
	    RETURN
	END IF

*   Begin by defining center using output from AXES.  If AXES has not been
*   run previously, then wait to see if the "CEN=" or "PROF" keywords have
*   been invoked.

	IF ( XCAX .EQ. 0.0 .AND. YCAX .EQ. 0.0 ) THEN
		CENTER = .FALSE.
	ELSE
		CENTER = .TRUE.
		R0 = YCAX
		C0 = XCAX
	END IF

*   Set the other default values
	PI = 3.1415926
	DCON = 1.745329e-2

C   Start with circular annuli
	INC = 0.0
	ECC = 0.0
	PA  = 0.0

	NROW = IMAGEER - IMAGESR + 1
	NCOL = IMAGEEC - IMAGESC + 1

	STEP = .FALSE.
C      annular spacing is 1 pixel
	DRANN = 1.0
	SCALED = .FALSE.
C      no user defined image scale
	ISCALE = 1.0
C      don`t use PROFILE output
	PROF = .FALSE.
        IPRFCOL = 1
C      use more than 1 annulus
	SINGLE = .FALSE.
C      use high accuracy SINC interpolation
	FAST = .FALSE.
        SILENT = .FALSE.

*  Read keywords

	DO 8701 I = 1, NCON

*       Number of annuli
		IF ( WORD(I)(1:2) .EQ. 'N=' ) THEN
			CALL ASSIGN ( WORD(I), F, PARM )
			IF (XERR) RETURN
			NANN = NINT(F)
			IF (NANN .LE. 0) THEN
			  PRINT *,'The number of annuli must be >0'
			  XERR    =.TRUE.
			  RETURN
			END IF

			IF ( NANN .GT. ANNMAX ) NANN = ANNMAX

*       Radial space between successive annuli
		ELSE IF ( WORD(I)(1:5) .EQ. 'STEP=' ) THEN
			CALL ASSIGN ( WORD(I), F, PARM )
			IF (XERR) RETURN
			DRANN = F
			STEP = .TRUE.

*       Position angle of the major axis of elliptical annuli in degrees
		ELSE IF ( WORD(I)(1:3) .EQ. 'PA=' ) THEN
			CALL ASSIGN ( WORD(I), F, PARM )
			IF (XERR) RETURN
			PA = F

*       Inclination angle of annuli in degrees (0 = face-on)
		ELSE IF ( WORD(I)(1:4) .EQ. 'INC=' ) THEN
			CALL ASSIGN ( WORD(I), F, PARM )
			IF (XERR) RETURN
			INC = F
			IF ( INC .GE. 90.0 ) THEN
			  PRINT *, 'Inclination must be < 90 degrees'
			  XERR = .TRUE.
			  RETURN
			ELSE IF ( INC. LT. 0.0 ) THEN
			  PRINT *, 'Inclination must be >= 0 degrees'
			  XERR = .TRUE.
			  RETURN
			END IF
			COSINC = COSD(INC)
			SININC = SIND(INC)

*       User defined center for annuli
		ELSE IF ( WORD(I)(1:4) .EQ. 'CEN=' ) THEN
			CALL ASSIGNV ( WORD(I), 2, G, NG, PARM)
			IF (XERR) RETURN
			IF ( NG .NE. 2) THEN
				PRINT *, 'CEN= needs 2 arguments'
				XERR = .TRUE.
				RETURN
			END IF
			CENTER = .TRUE.
			R0 = G(1)
			C0 = G(2)
			IF ( R0 .GT. FLOAT(IMAGEER) .OR.
     &                       R0 .LT. FLOAT(IMAGESR) ) THEN
		          PRINT *, 'R0 outside image bounds'
			  PRINT 
     &                      '(1X,A,I5,A,I5,A)','(',IMAGESR,',',IMAGEER,')'
			  XERR = .TRUE.
			  RETURN
			END IF
			IF ( C0 .GT. FLOAT(IMAGEEC) .OR.
     &                       C0 .LT. FLOAT(IMAGESC) ) THEN
				PRINT *, 'C0 outside image bounds'
				PRINT 
     &                      '(1X,A,I5,A,I5,A)','(',IMAGESC, ',',IMAGEEC,')'
				XERR = .TRUE.
				RETURN
			END IF

*       Image Scale
		ELSE IF ( WORD(I)(1:6) .EQ. 'SCALE=' ) THEN
			CALL ASSIGN ( WORD(I), F, PARM )
			IF (XERR) RETURN
			ISCALE = F
			IF ( ISCALE .LE. 0.0 ) THEN
				PRINT *, 'SCALE must be >= 0.0'
				XERR = .TRUE.
				RETURN
			END IF
			SCALED = .TRUE.

*       Use faster bi-linear interpolation in outer annuli
		ELSE IF ( WORD(I) .EQ. 'FAST' ) THEN
			FAST = .TRUE.

*       Use the results of a previous PROFILE calculation to define
*       the annuli.
		ELSE IF ( WORD(I) .EQ. 'PROF' ) THEN
			PROF = .TRUE.
			CENTER = .TRUE.
			ISCALE = PRFSC(1)
			SCALED = .TRUE.
                        IPRFCOL = 1

		ELSE IF ( WORD(I)(1:5) .EQ. 'PROF=' ) THEN
                        CALL ASSIGN(WORD(I),F,PARM)
                        IF (XERR) RETURN
                        IPRFCOL = NINT(F)
			PROF = .TRUE.
			CENTER = .TRUE.
			ISCALE = PRFSC(IPRFCOL)
			SCALED = .TRUE.

*       Only average along a single, given annulus
		ELSE IF ( WORD(I)(1:4) .EQ. 'RAD=' ) THEN
			SINGLE = .TRUE.
			CALL ASSIGN ( WORD(I), F, PARM )
			IF (XERR) RETURN
			RANN = F

                ELSE IF (WORD(I) .EQ. 'SILENT') THEN
                        SILENT = .TRUE.

		END IF

8701    CONTINUE

*   If the user has not defined an object center, either explicitly with the
*   "CEN=" keyword, or by previous execution of either AXES or PROFILE, then
*   inform the user of this and exit gracefully

	IF ( .NOT. CENTER ) THEN
		PRINT *, ' * * * No Object Center has been defined * * *'
		PRINT *, 'Either run AXES or use the CEN= keyword'
		PRINT *, '(or use PROFILE or a SAVEd profile calculation)'
		XERR = .TRUE.
		RETURN
	END IF

*  Check to see (if relevant) that the spectrum that is being created will be
*  in a different buffer from the image.

	IF ( .NOT. SINGLE ) THEN
	    IF ( IBUF(1) .EQ. IBUF(2) ) THEN
		PRINT *,'The buffer for the destination spectrum must'
		PRINT *,'be different from the one holding the image.'
		XERR = .TRUE.
		RETURN
	    END IF
	END IF

*   Establish the spacing between annuli.  If the "SCALE=" and "STEP=" keywords
*   were given, then convert the user defined annular spacing to pixels.  If
*   the "SCALE=" keyword was not used, DRANN is already in units of pixels.

	IF ( SCALED .AND. STEP ) DRANN = DRANN / ISCALE

*   Print calculation header.

	IF (.NOT. SILENT) THEN

       WRITE(olu, '(/1x,''* * * AZIMUTHAL AVERAGE CALCULATION * * *'')')

	IF (SINGLE) THEN
            WRITE(olu, 101) R0, C0, RANN*ISCALE
101         FORMAT(/1X,'Single Annulus centered at (',f6.2,',',
     &              f7.2,'), with Radius =',f7.2 ) 
	    IF (INC .NE. 0.0 ) 
     &         WRITE(olu, 102) PA, INC
102            FORMAT(1X,'PA = ',f7.2,'degrees',
     &                        ', Inclination = ',f5.2,' degrees')
	ELSE IF (PROF) THEN
	    WRITE(olu, '(/1X,''Annuli defined from PROFILE fit'')' )
	ELSE
	    WRITE(olu, 103) R0, C0
103         FORMAT(/1X,'Annuli centered at (',f6.2,',',f7.2, ')') 
	    IF (SCALED) THEN
		WRITE(olu, 104) DRANN*ISCALE, DRANN
104             FORMAT(1X,'Annular spacing = ',f7.2, 
     &                 ' Arcsec [',f7.2,' pixel(s)]') 
	    ELSE
		WRITE(olu, 105) DRANN
105             FORMAT(1X,'Annular spacing = ',f7.2,' Pixel(s)') 
	    END IF

	    IF (INC .NE. 0.0) WRITE(olu, 106) PA, INC
106         FORMAT(1X,'PA = ',f7.2,' degrees',
     &            ', Inclination = ',f5.2,' degrees') 
	END IF

        END IF

C    Initialize interpolation routines

	D = BINSET(A,NROW,NCOL,.FALSE.)
********************************************************************************
*                                                                              *
*       In this section we do the azimuthal averaging along annuli             *
*                                                                              *
********************************************************************************

*   CASE I:  a single annulus (simple)

	IF (SINGLE) THEN

	    IF ( RANN .LE. 2.0 ) THEN
		NSAMP = 8*PI*RANN + 4
	    ELSE
		NSAMP = 2*PI*RANN + 1
	    END IF

	    DTHETA = 2.0*PI / FLOAT(NSAMP)

	    AZAVG(1) = 0.0
	    NACTUAL = NSAMP

	    DO 8702 I = 1, NSAMP

		IF (NOGO) RETURN

		THETA = FLOAT(I)*DTHETA
		COSTH = COS(THETA)
		SINTH = SIN(THETA)

		IF ( INC .EQ. 0.0 ) THEN

		    RSAMP = R0 - ( RANN * COSTH )
		    CSAMP = C0 - ( RANN * SINTH )

		ELSE

		    PHI = DCON*PA + THETA
		    RPROJ = RANN*COSINC/SQRT(1.0-(SININC*COSTH)**2)
		    RSAMP = R0 - ( RPROJ * COS( PHI ) )
		    CSAMP = C0 - ( RPROJ * SIN( PHI ) )

		END IF

		IRS = NINT(RSAMP)
		ICS = NINT(CSAMP)

		IF ( IRS .GE. IMAGESR
     &               .AND. IRS .LE. IMAGEER
     &               .AND. ICS .GE. IMAGESC
     &               .AND. ICS .LE. IMAGEEC ) THEN

*       Interpolation routines have images dimensioned as (1:NCOL,1:NROW)

		    RSAMP = RSAMP - FLOAT(IMAGESR) + 1.0
		    CSAMP = CSAMP - FLOAT(IMAGESC) + 1.0

		    IF (RANN .LE. 15.0) THEN
C                      Use 2D sinc interpolation
		      ISAMP = XBIN(CSAMP,RSAMP,A,NROW,NCOL,.FALSE.)
		    ELSE IF ( FAST ) THEN
C                      Use bilinear interpolation
		      ISAMP = BIN(CSAMP,RSAMP,A,NROW,NCOL,.FALSE.)
		    ELSE
C                      Use shorter sinc
		      ISAMP = OUTBIN(CSAMP,RSAMP,A,NROW,NCOL,.FALSE.)
		    ENDIF

		    AZAVG(1) = ISAMP + AZAVG(1)

		ELSE
C                 Sample point falls off image

		    NACTUAL = NACTUAL - 1

		END IF

8702        CONTINUE

	    IF ( NACTUAL .LT. 1 ) THEN

		    PRINT *, 'Given Annulus falls completely outside image'
		    XERR = .TRUE.
		    RETURN

	    ELSE IF ( NACTUAL .LT. NSAMP ) THEN

		    PLOST = 100.0*FLOAT(NACTUAL)/FLOAT(NSAMP)
		    IF (.NOT. SILENT) WRITE(olu,107) PLOST
107                 FORMAT(1X,'*** WARNING: Given Annulus is only',
     &                   f5.1,'% complete.')

	    END IF

	    AZAVG(1) = AZAVG(1) / FLOAT(NACTUAL)

	    IF (.NOT. SILENT) 
     &     WRITE(olu, '(/1X,''Azimuthal Average = '',1pg14.6)') AZAVG(1)

	    RETURN

********************************************************************************

*   CASE II:  using contours from a previous PROFILE calculation

	ELSE IF (PROF) THEN

*       Output Format

200         FORMAT (2X,F7.2,3X,F7.2,3X,F7.2,3X,F7.2,3X,F5.3,3X,1PG14.6)

*       Write Header Information

	    IF (.NOT. SILENT) THEN
              WRITE(olu, 108)
108           FORMAT(/1X,T4,'R (")',T16,'R0',T26,'C0',T36,'PA',
     &                       T44,'Ecc',T54,'Average')
              WRITE(olu, 109)
109           FORMAT(1X,'--------------------',
     &                  '--------------------',
     &                  '--------------------','-----')
            END IF

*       First value is "average" at center

            NANN = NPRF(IPRFCOL) - 1
	    RSAMP = PARAMPRF(1,1,IPRFCOL) - FLOAT(IMAGESR) + 1.0
	    CSAMP = PARAMPRF(2,1,IPRFCOL) - FLOAT(IMAGESC) + 1.0

	    AZAVG(1) = XBIN ( CSAMP, RSAMP, A, NROW, NCOL,.FALSE.)

	    IF (.NOT. SILENT) 
     &          WRITE(olu, 200) 0.0, RSAMP, CSAMP, 0.0, 0.0, AZAVG(1)

*       Do each contour in turn

	    DO 8703 J = 2, NPRF(IPRFCOL)

		IF (NOGO) RETURN

*       Grab relevant parameters of each contour

		 R0 = PARAMPRF(1,J,IPRFCOL)
		 C0 = PARAMPRF(2,J,IPRFCOL)
		 PA = PARAMPRF(3,J,IPRFCOL)
		ECC = PARAMPRF(4,J,IPRFCOL)

		RANN = FLOAT(J)

		   B = RANN * SQRT(1.0 - ECC**2)

		IF ( RANN .LE. 2.0 ) THEN
		    NSAMP = 8*PI*RANN + 4
		ELSE
		    NSAMP = 2*PI*RANN + 1
		END IF

		DTHETA = 2.0*PI / FLOAT(NSAMP)

		AZAVG(J) = 0.0
		NACTUAL = NSAMP

		DO 8704 I = 1, NSAMP

		    IF (NOGO) RETURN

		    THETA = FLOAT(I)*DTHETA
		    COSTH = COS(THETA)
		    SINTH = SIN(THETA)

		    IF ( ECC .EQ. 0.0 ) THEN

			RSAMP = R0 - ( RANN * COSTH )
			CSAMP = C0 - ( RANN * SINTH )

		    ELSE

			PHI = DCON*PA + THETA
			RPROJ = B / SQRT(1.0 - (ECC*COSTH)**2)
			RSAMP = R0 - ( RPROJ * COS( PHI ) )
			CSAMP = C0 - ( RPROJ * SIN( PHI ) )

		    END IF

		    IRS = NINT(RSAMP)
		    ICS = NINT(CSAMP)

		    IF ( IRS .GE. IMAGESR
     &                   .AND. IRS .LE. IMAGEER
     &                   .AND. ICS .GE. IMAGESC
     &                   .AND. ICS .LE. IMAGEEC ) THEN

*       Interpolation routines have images dimensioned as (1:NCOL,1:NROW)

			RSAMP = RSAMP - FLOAT(IMAGESR) + 1.0
			CSAMP = CSAMP - FLOAT(IMAGESC) + 1.0

			IF (RANN .LE. 15.0) THEN
			  ISAMP = XBIN(CSAMP,RSAMP,A,NROW,NCOL,.FALSE.)
			ELSE IF ( FAST ) THEN
			  ISAMP = BIN(CSAMP,RSAMP,A,NROW,NCOL,.FALSE.)
			ELSE
			  ISAMP = OUTBIN(CSAMP,RSAMP,A,NROW,NCOL,.FALSE.)
			ENDIF
			AZAVG(J) = ISAMP + AZAVG(J)

		    ELSE
C                     Sample point falls off image
			NACTUAL = NACTUAL - 1

		    END IF

8704            CONTINUE

		IF ( NACTUAL .LT. 1 ) THEN

		    AZAVG(J) = 0.0

                    IF (.NOT. SILENT) THEN
		    WRITE(olu, 200) RANN*ISCALE, R0, C0, PA, ECC,
     &                    AZAVG(J)
		    WRITE(olu,110)
110                 FORMAT(4X,'* * *',1x,
     &              'Annulus falls completely outside image * * * ')
                    END IF

		ELSE IF ( NACTUAL .LT. NSAMP .AND. NACTUAL .GE. 1 )
     &                THEN

		    AZAVG(J) = AZAVG(J) / FLOAT(NACTUAL)

		    PLOST = 100.0*FLOAT(NACTUAL)/FLOAT(NSAMP)
                    IF (.NOT. SILENT) THEN
		      WRITE(olu, 200) RANN*ISCALE, R0, C0, PA, ECC,
     &                    AZAVG(J)
		      WRITE(olu,111) PLOST
111                   FORMAT(1X,'*** WARNING: Annulus is only ',
     &                   f5.1,'% complete ***')
                    END IF

		ELSE

		    AZAVG(J) = AZAVG(J) / FLOAT(NACTUAL)

		    IF (.NOT. SILENT) 
     &                 WRITE(olu, 200) RANN*ISCALE, R0, C0, PA, ECC,AZAVG(J)

		END IF

*       Go get the next contour level until all gone

8703        CONTINUE

********************************************************************************

*   CASE III:  Concentric Annuli defined by user inputs

	ELSE

*       Output Format

300         FORMAT (2X,F7.2,3X,1PG14.6)

*       Write Header Information

            IF (.NOT. SILENT) THEN
	      WRITE(olu, '(/1X,T4,''R (")'',T16,''Average'')')
	      WRITE(olu, '(1X,''---------------------------'')' )
            END IF

*       First value is "average" at RANN=0.0 (at center)

	    RSAMP = R0 - FLOAT(IMAGESR) + 1.0
	    CSAMP = C0 - FLOAT(IMAGESC) + 1.0

	    AZAVG(1) = XBIN ( CSAMP, RSAMP, A, NROW, NCOL, .FALSE. )

	    IF (.NOT. SILENT) WRITE(olu, 300 ) 0.0, AZAVG(1)

*       Do each annulus in turn

	    DO 8705 J = 2, NANN

		IF (NOGO) RETURN

		RANN = FLOAT(J-1)*DRANN

		IF ( RANN .LE. 2.0 ) THEN
		    NSAMP = 8*PI*RANN + 4
		ELSE
		    NSAMP = 2*PI*RANN + 1
		END IF

		DTHETA = 2.0*PI / FLOAT(NSAMP)

		AZAVG(J) = 0.0
		NACTUAL = NSAMP

		DO 8706 I = 1, NSAMP

		    IF (NOGO) RETURN

		    THETA = FLOAT(I)*DTHETA
		    COSTH = COS(THETA)
		    SINTH = SIN(THETA)

		    IF ( INC .EQ. 0.0 ) THEN

			RSAMP = R0 - ( RANN * COSTH )
			CSAMP = C0 - ( RANN * SINTH )

		    ELSE

			PHI = DCON*PA + THETA
			RPROJ = RANN*COSINC/SQRT(1.0-(SININC*COSTH)**2)
			RSAMP = R0 - ( RPROJ * COS( PHI ) )
			CSAMP = C0 - ( RPROJ * SIN( PHI ) )

		    END IF

		    IRS = NINT(RSAMP)
		    ICS = NINT(CSAMP)

		    IF ( IRS .GE. IMAGESR
     &                   .AND. IRS .LE. IMAGEER
     &                   .AND. ICS .GE. IMAGESC
     &                   .AND. ICS .LE. IMAGEEC ) THEN

*       Interpolation routines have images dimensioned as (1:NCOL,1:NROW)

			RSAMP = RSAMP - FLOAT(IMAGESR) + 1.0
			CSAMP = CSAMP - FLOAT(IMAGESC) + 1.0

			IF (RANN .LE. 15.0) THEN
			  ISAMP = XBIN(CSAMP,RSAMP,A,NROW,NCOL,.FALSE.)
			ELSE IF ( FAST ) THEN
			  ISAMP = BIN(CSAMP,RSAMP,A,NROW,NCOL,.FALSE.)
			ELSE
			  ISAMP = OUTBIN(CSAMP,RSAMP,A,NROW,NCOL,.FALSE.)
			ENDIF
			AZAVG(J) = ISAMP + AZAVG(J)

		    ELSE
C                     Sample point falls off image

			NACTUAL = NACTUAL - 1

		    END IF

8706            CONTINUE

		IF ( NACTUAL .LT. 1 ) THEN

		    AZAVG(J) = 0.0

                    IF (.NOT. SILENT) THEN
		      WRITE(olu, 300 ) RANN*ISCALE, AZAVG(J)
		      WRITE(olu,112)
112                   FORMAT(4X,'* * *',1x,
     &                       'Annulus falls completely outside image * * * ')
                    END IF

		ELSE IF ( NACTUAL .LT. NSAMP .AND. NACTUAL .GE. 1 )
     &                THEN

		    AZAVG(J) = AZAVG(J) / FLOAT(NACTUAL)

		    PLOST = 100.0*FLOAT(NACTUAL)/FLOAT(NSAMP)
                    IF (.NOT. SILENT) THEN
		      WRITE(olu, 300 ) RANN*ISCALE, AZAVG(J)
		      WRITE(olu,113) PLOST
113                   FORMAT(1X,'*** WARNING: Annulus is only ',
     &                       f5.1,'% complete ***')
                    END IF

		ELSE

		    AZAVG(J) = AZAVG(J) / FLOAT(NACTUAL)
		    IF (.NOT. SILENT) WRITE(olu, 300 ) RANN*ISCALE, AZAVG(J)

		END IF

8705        CONTINUE

	END IF

*   Create the spectrum with the azimuthal averages in the specified destination
*   buffer.  If the SINGLE annulus option has been invoked, skip this step
*   and exit.

	IF ( SINGLE ) RETURN

*   Load the appropriate FITS header cards

	IF ( SCALED .AND. STEP ) THEN
		DELT1 = DBLE(DRANN * ISCALE)
	ELSE IF ( SCALED .AND. .NOT. STEP ) THEN
		DELT1 = DBLE(ISCALE)
	ELSE
		DELT1 = DBLE(DRANN)
	END IF
	TEMPHEAD = ' '
C     image FITS header
	CALL CCVSTRCPY(TEMPHEAD,HEADBUF(1,IM))
C     a 1-D image
	CALL INHEADSET('NAXIS2',1,TEMPHEAD)
C     # of annuli
	CALL INHEADSET('NAXIS1',NANN,TEMPHEAD)
	CALL FHEADSET('CRVAL1',0.0D0,TEMPHEAD)
C     annular spacing
	CALL FHEADSET('CDELT1',DELT1,TEMPHEAD)
	CALL INHEADSET('CRPIX1',1,TEMPHEAD)
	IF ( SCALED ) THEN
		CALL CHEADSET('CTYPE1','Radius (Arcsec)',TEMPHEAD)
	ELSE
		CALL CHEADSET('CTYPE1','Radius (Pixels)',TEMPHEAD)
	END IF
	CALL CHEADSET('BUNIT','Intensity',TEMPHEAD)
	CALL CHEADSET('HISTORY','Output from ANNULUS',TEMPHEAD)
	CALL CHEADSET('STATUS','Azimuthal Average',TEMPHEAD)

	CALL CREATEIM(LOCNEW,IMNEWSR,IMNEWER,
     &                IMNEWSC,IMNEWEC,1,TEMPHEAD,.TRUE.)

	IF (XERR) RETURN

	CALL CCCOPIO (LOCNEW, 1, NANN, AZAVG, 1, NANN, 0, 0 )

	RETURN
	END
